import { Directive, Input, Output, EventEmitter, HostBinding, ElementRef, NgZone, Renderer2, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class ResizedEvent {
    /**
     * @param {?} newWidth
     * @param {?} newHeight
     * @param {?} oldWidth
     * @param {?} oldHeight
     */
    constructor(newWidth, newHeight, oldWidth, oldHeight) {
        this.newWidth = newWidth;
        this.newHeight = newHeight;
        this.oldWidth = oldWidth;
        this.oldHeight = oldHeight;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class AutoScrollToBottomDirective {
    /**
     * @param {?} element
     * @param {?} zone
     * @param {?} renderer
     */
    constructor(element, zone, renderer) {
        this.element = element;
        this.zone = zone;
        this.renderer = renderer;
        this.autoScroll = true;
        this.autoScrollChange = new EventEmitter();
        this.resized = new EventEmitter();
        this.overflowAnchor = 'none';
        this.overflowY = 'auto';
        this.scrolling = false;
        this.onScroll = () => {
            /** @type {?} */
            const maxDelta = 2;
            if (this.scrolling) {
                return;
            }
            /** @type {?} */
            const el = this.element.nativeElement;
            /** @type {?} */
            const bottom = el.scrollTop + el.offsetHeight;
            /** @type {?} */
            const height = el.scrollHeight;
            /** @type {?} */
            const atBottom = height - bottom < maxDelta;
            if (this.autoScroll !== atBottom) {
                this.zone.run(() => {
                    this.autoScroll = atBottom;
                    this.autoScrollChange.emit(this.autoScroll);
                });
            }
        };
        this.scrollToBottom = (event) => {
            if (!this.autoScroll) {
                return;
            }
            /** @type {?} */
            const el = this.element.nativeElement;
            setTimeout(() => {
                el.scrollTop = el.scrollHeight - el.clientHeight;
            });
            this.scrolling = true;
            this.zone.runOutsideAngular(() => setTimeout(() => this.scrolling = false, 1000));
        };
        this.scrollToBottomPreset = this.scrollToBottom;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.zone.runOutsideAngular(() => {
            this.renderer.listen(this.element.nativeElement, 'scroll', () => this.onScroll());
        });
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        setTimeout(() => {
            // scroll to bottom at the beginning
            this.scrollToBottom(null);
            /** @type {?} */
            const el = this.element.nativeElement;
            /** @type {?} */
            const wrapper = document.createElement('div');
            wrapper.setAttribute('style', 'position: relative;');
            /** @type {?} */
            const childNodes = el.childNodes;
            /** @type {?} */
            const length = childNodes.length;
            for (let i = 0; i < length; i++) {
                /** @type {?} */
                const node = childNodes.item(0);
                wrapper.appendChild(node);
            }
            el.appendChild(wrapper);
            this.resized.subscribe((event) => {
                this.scrollToBottomPreset(event);
            });
            this.onResizeFn(wrapper, (oldVal, newVal) => {
                this.resized.emit(new ResizedEvent(newVal.width, newVal.height, oldVal.width, oldVal.height));
            });
        });
    }
    /**
     * @param {?} el
     * @param {?} callback
     * @return {?}
     */
    onResizeFn(el, callback) {
        /** @type {?} */
        const iframe = document.createElement('iframe');
        iframe.setAttribute('style', `
      width: 100%;
      height: 100%;
      position: absolute;
      visibility:hidden;
      margin: 0;
      padding: 0;
      border: 0;
      left: 0;
      top: 0;
      `);
        el.appendChild(iframe);
        /** @type {?} */
        let oldWidth = el.offsetWidth;
        /** @type {?} */
        let oldHeight = el.offsetHeight;
        /** @type {?} */
        const sizeChange = () => {
            /** @type {?} */
            const width = el.offsetWidth;
            /** @type {?} */
            const height = el.offsetHeight;
            // 不一致时触发回调函数 callback，并更新元素当前宽高
            if (width !== oldWidth || height !== oldHeight) {
                callback({ width: width, height: height }, { width: oldWidth, height: oldHeight });
                oldWidth = width;
                oldHeight = height;
            }
        };
        /** @type {?} */
        let timer = null;
        // 将 sizeChange 函数挂载到 iframe 的resize回调中
        iframe.contentWindow.onresize = () => {
            clearTimeout(timer);
            timer = setTimeout(sizeChange, 33);
        };
    }
}
AutoScrollToBottomDirective.decorators = [
    { type: Directive, args: [{
                selector: '[appAutoScrollToBottom]',
                exportAs: 'autoScrollDirective'
            },] }
];
/** @nocollapse */
AutoScrollToBottomDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgZone },
    { type: Renderer2 }
];
AutoScrollToBottomDirective.propDecorators = {
    autoScroll: [{ type: Input }],
    scrollToBottomPreset: [{ type: Input }],
    autoScrollChange: [{ type: Output }],
    resized: [{ type: Output }],
    overflowAnchor: [{ type: HostBinding, args: ['style.overflow-anchor',] }],
    overflowY: [{ type: HostBinding, args: ['style.overflow-y',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class AngularAutoScrollToBottomModule {
}
AngularAutoScrollToBottomModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                declarations: [AutoScrollToBottomDirective],
                exports: [AutoScrollToBottomDirective]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

export { AngularAutoScrollToBottomModule, ResizedEvent, AutoScrollToBottomDirective as ɵa };

//# sourceMappingURL=angular-auto-scroll-to-bottom.js.map